// meshenum.cpp : ask for a mesh file and dump it out to the debug output
//
// Copyright (C) 2000-2001, Rich Thomson, all rights reserved.
//
#include "stdafx.h"

//////////////////////////////////////////////////////////////////////

const int FRAME_WIDTH = 320;
const int FRAME_HEIGHT = 240;

CComPtr<IDirect3DDevice8> g_device;

// device presentation parameters
static D3DPRESENT_PARAMETERS g_presentation = {
    // width, height, format, number of back buffers
    FRAME_WIDTH, FRAME_HEIGHT, D3DFMT_UNKNOWN, 1,
    // multisample type, swap effect, device window, windowed
    D3DMULTISAMPLE_NONE, D3DSWAPEFFECT_FLIP, HWND(0), TRUE,
    // auto depth/stencil surface, depth/stencil surface format
    false, D3DFMT_UNKNOWN
};

static const HWND &g_window = g_presentation.hDeviceWindow;

class c_window
{
public:
    c_window(HWND window) : m_window(window)
    {}
    ~c_window()
    {
        ::DestroyWindow(m_window);
    }
    operator HWND() const
    {
        return m_window;
    }

private:
    HWND m_window;
};

HWND
create_window(HINSTANCE instance)
{
    // Register the window class for the main window.
    const TCHAR APP_NAME[] = _T("Mesh Enum");
    const WNDCLASS wc = {
        0, ::DefWindowProc, 0, 0, instance,
        NULL, LoadCursor(NULL, IDC_CROSS),
#if defined(_DEBUG)
        CreateHatchBrush(HS_FDIAGONAL, RGB(255, 255, 0)),
#else
        static_cast<HBRUSH>(GetStockObject(BLACK_BRUSH)),
#endif
        NULL, APP_NAME
    };
    TWS(::RegisterClass(&wc));

    // Create the main window.
    return
    TWS(::CreateWindow(APP_NAME, APP_NAME, WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       FRAME_WIDTH, FRAME_HEIGHT,
                       GetDesktopWindow(), NULL, instance, NULL));
}

int __stdcall
_tWinMain(HINSTANCE instance, HINSTANCE, LPTSTR, int)
{
    try
    {
        rt::com_attach_ptr<IDirect3D8> d3d(Direct3DCreate8(D3D_SDK_VERSION));
        c_window device_window(create_window(instance));
        D3DDISPLAYMODE dm;
        THR(d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &dm));
        g_presentation.hDeviceWindow = device_window;
        g_presentation.BackBufferFormat = dm.Format;
        THR(d3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
                              device_window,
                              D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                              &g_presentation, &g_device));

        TCHAR mesh_file[MAX_PATH] = { 0 };
        OPENFILENAME ofn =
        {
            sizeof(ofn), 0, 0, _T("Mesh Files (*.x)\0*.x\0"), 0, 0, 0,
            mesh_file, MAX_PATH, 0, 0, 0, 0, OFN_FILEMUSTEXIST
        };
        if (::GetOpenFileName(&ofn))
        {
            dump_mesh(g_device, mesh_file);
        }
    }
    catch (rt::hr_message &bang)
    {
        return rt::display_error(bang);
    }

	return 0;
}
